@@ -18,7 +18,7 @@
  *
  **********************************************************************
  *
- * Copyright (C) 2015 Sandro Santilli <strk@kbt.io>
+ * Copyright (C) 2015-2017 Sandro Santilli <strk@kbt.io>
  *
  **********************************************************************/
 
@@ -188,6 +188,7 @@ typedef struct LWT_BE_CALLBACKS_T {
    * @param ids an array of element identifiers
    * @param numelems input/output parameter, pass number of node identifiers
    *                 in the input array, gets number of node in output array.
+   *	TODO: Should be uint64 to match SPI_processed
    * @param fields fields to be filled in the returned structure, see
    *               LWT_COL_NODE_* macros
    *
@@ -212,6 +213,7 @@ typedef struct LWT_BE_CALLBACKS_T {
    * @param numelems output parameter, gets number of elements found
    *                 if the return is not null, otherwise see @return
    *                 section for semantic.
+   *	TODO: Should be uint64 to match SPI_processed
    * @param fields fields to be filled in the returned structure, see
    *               LWT_COL_NODE_* macros
    * @param limit max number of nodes to return, 0 for no limit, -1
@@ -239,6 +241,7 @@ typedef struct LWT_BE_CALLBACKS_T {
    * @param nodes the nodes to insert. Those with a node_id set to -1
    *              it will be replaced to an automatically assigned identifier
    * @param nelems number of elements in the nodes array
+   *	TODO: Should be uint64 to match SPI_processed
    *
    * @return 1 on success, 0 on error (@see lastErrorMessage)
    */
@@ -281,7 +284,7 @@ typedef struct LWT_BE_CALLBACKS_T {
    * @param fields fields to be filled in the returned structure, see
    *               LWT_COL_EDGE_* macros
    * @param limit max number of edges to return, 0 for no limit, -1
-   *              to only check for existance if a matching row.
+   *              to only check for existence if a matching row.
    *
    * @return an array of edges or null in the following cases:
    *         - limit=-1 ("numelems" is set to 1 if found, 0 otherwise)
@@ -319,6 +322,7 @@ typedef struct LWT_BE_CALLBACKS_T {
    * @param edges the edges to insert. Those with a edge_id set to -1
    *              it will be replaced to an automatically assigned identifier
    * @param nelems number of elements in the edges array
+   *	TODO: Should be uint64 to match SPI_processed
    *
    * @return number of inserted edges, or -1 (@see lastErrorMessage)
    */
@@ -439,7 +443,7 @@ typedef struct LWT_BE_CALLBACKS_T {
    * @param fields fields to be filled in the returned structure, see
    *               LWT_COL_NODE_* macros
    * @param limit max number of nodes to return, 0 for no limit, -1
-   *              to only check for existance if a matching row.
+   *              to only check for existence if a matching row.
    *
    * @return an array of nodes or null in the following cases:
    *         - limit=-1 ("numelems" is set to 1 if found, 0 otherwise)
@@ -457,14 +461,14 @@ typedef struct LWT_BE_CALLBACKS_T {
    * Get edges whose bounding box overlaps a given 2D bounding box
    *
    * @param topo the topology to act upon
-   * @param box the query box
+   * @param box the query box, to be considered infinite if NULL
    * @param numelems output parameter, gets number of elements found
    *                 if the return is not null, otherwise see @return
    *                 section for semantic.
    * @param fields fields to be filled in the returned structure, see
    *               LWT_COL_EDGE_* macros
    * @param limit max number of edges to return, 0 for no limit, -1
-   *              to only check for existance if a matching row.
+   *              to only check for existence if a matching row.
    *
    * @return an array of edges or null in the following cases:
    *         - limit=-1 ("numelems" is set to 1 if found, 0 otherwise)
@@ -512,7 +516,7 @@ typedef struct LWT_BE_CALLBACKS_T {
    * @param upd_fields fields to be updated for the selected nodes,
    *                   see LWT_COL_NODE_* macros
    * @param exc_node an LWT_ISO_NODE object with exclusion fields set,
-   *                 can be NULL if no exlusion condition exists.
+   *                 can be NULL if no exclusion condition exists.
    * @param exc_fields fields used for excluding nodes from the update,
    *                   see LWT_COL_NODE_* macros
    *
@@ -555,6 +559,7 @@ typedef struct LWT_BE_CALLBACKS_T {
    * @param faces the faces to insert. Those with a node_id set to -1
    *              it will be replaced to an automatically assigned identifier
    * @param nelems number of elements in the faces array
+   *	TODO: Should be uint64 to match SPI_processed
    *
    * @return number of inserted faces, or -1 (@see lastErrorMessage)
    */
@@ -571,6 +576,7 @@ typedef struct LWT_BE_CALLBACKS_T {
    * @param faces an array of LWT_ISO_FACE object with selecting id
    *              and setting mbr.
    * @param numfaces number of faces in the "faces" array
+   *	TODO: Should be uint64 to match SPI_processed
    *
    * @return number of faces being updated or -1 on error
    *         (@see lastErroMessage)
@@ -598,13 +604,14 @@ typedef struct LWT_BE_CALLBACKS_T {
    *             side (left if positive, right if negative)
    *             and direction (forward if positive, backward if negative).
    * @param numedges output parameter, gets the number of edges visited
+   *
    * @param limit max edges to return (to avoid an infinite loop in case
    *              of a corrupted topology). 0 is for unlimited.
    *              The function is expected to error out if the limit is hit.
    *
    * @return an array of signed edge identifiers (positive edges being
    *         walked in their direction, negative ones in opposite) or
-   *         NULL on error (@see lastErroMessage)
+   *         NULL on error (@see lastErrorMessage)
    */
   LWT_ELEMID* (*getRingEdges) (
       const LWT_BE_TOPOLOGY* topo,
@@ -618,11 +625,12 @@ typedef struct LWT_BE_CALLBACKS_T {
    * @param edges an array of LWT_ISO_EDGE object with selecting id
    *              and updating fields.
    * @param numedges number of edges in the "edges" array
+   *	TODO: Should be uint64 to match SPI_processed
    * @param upd_fields fields to be updated for the selected edges,
    *                   see LWT_COL_EDGE_* macros
    *
    * @return number of edges being updated or -1 on error
-   *         (@see lastErroMessage)
+   *         (@see lastErrorMessage)
    */
   int (*updateEdgesById) (
       const LWT_BE_TOPOLOGY* topo,
@@ -641,6 +649,7 @@ typedef struct LWT_BE_CALLBACKS_T {
    *                 in the input array, gets number of edges in output array
    *                 if the return is not null, otherwise see @return
    *                 section for semantic.
+   *	TODO: Should be uint64 to match SPI_processed
    * @param fields fields to be filled in the returned structure, see
    *               LWT_COL_EDGE_* macros
    * @param box optional bounding box to further restrict matches, use
@@ -665,6 +674,7 @@ typedef struct LWT_BE_CALLBACKS_T {
    *                 identifiers in the input array, gets number of
    *                 nodes in output array if the return is not null,
    *                 otherwise see @return section for semantic.
+   *	TODO: Should be uint64 to match SPI_processed
    * @param fields fields to be filled in the returned structure, see
    *               LWT_COL_NODE_* macros
    * @param box optional bounding box to further restrict matches, use
@@ -687,11 +697,12 @@ typedef struct LWT_BE_CALLBACKS_T {
    * @param nodes an array of LWT_ISO_EDGE objects with selecting id
    *              and updating fields.
    * @param numnodes number of nodes in the "nodes" array
+   *	TODO: Should be uint64 to match SPI_processed
    * @param upd_fields fields to be updated for the selected edges,
    *                   see LWT_COL_NODE_* macros
    *
    * @return number of nodes being updated or -1 on error
-   *         (@see lastErroMessage)
+   *         (@see lastErrorMessage)
    */
   int (*updateNodesById) (
       const LWT_BE_TOPOLOGY* topo,
@@ -705,9 +716,10 @@ typedef struct LWT_BE_CALLBACKS_T {
    * @param topo the topology to act upon
    * @param ids an array of face identifiers
    * @param numelems number of face identifiers in the ids array
+   *	TODO: Should be uint64 to match SPI_processed
    *
    * @return number of faces being deleted or -1 on error
-   *         (@see lastErroMessage)
+   *         (@see lastErrorMessage)
    */
   int (*deleteFacesById) (
       const LWT_BE_TOPOLOGY* topo,
@@ -744,9 +756,10 @@ typedef struct LWT_BE_CALLBACKS_T {
    * @param topo the topology to act upon
    * @param ids an array of node identifiers
    * @param numelems number of node identifiers in the ids array
+   *	TODO: Should be uint64 to match SPI_processed
    *
    * @return number of nodes being deleted or -1 on error
-   *         (@see lastErroMessage)
+   *         (@see lastErrorMessage)
    */
   int (*deleteNodesById) (
       const LWT_BE_TOPOLOGY* topo,
@@ -847,10 +860,11 @@ typedef struct LWT_BE_CALLBACKS_T {
    * @param numelems output parameter, gets number of elements found
    *                 if the return is not null, otherwise see @return
    *                 section for semantic.
+   *	TODO: Should be uint64 to match SPI_processed
    * @param fields fields to be filled in the returned structure, see
    *               LWT_COL_FACE_* macros
    * @param limit max number of faces to return, 0 for no limit, -1
-   *              to only check for existance if a matching row.
+   *              to only check for existence if a matching row.
    *
    * @return an array of faces or null in the following cases:
    *         - limit=-1 ("numelems" is set to 1 if found, 0 otherwise)
@@ -1056,6 +1070,60 @@ LWT_ELEMID lwt_AddPoint(LWT_TOPOLOGY* topo, LWPOINT* point, double tol);
 LWT_ELEMID* lwt_AddLine(LWT_TOPOLOGY* topo, LWLINE* line, double tol,
                         int* nedges);
 
+/**
+ * Adds a linestring to the topology without determining generated faces
+ *
+ * The given line will snap to existing nodes or edges within given
+ * tolerance. Existing edges or faces may be split by the line.
+ *
+ * Side faces for the new edges will not be determined and no new
+ * faces will be created, effectively leaving the topology in an
+ * invalid state (WARNING!)
+ *
+ * @param topo the topology to operate on
+ * @param line the line to add
+ * @param tol snap tolerance, the topology tolerance will be used if 0
+ * @param nedges output parameter, will be set to number of edges the
+ *               line was split into, or -1 on error
+ *               (liblwgeom error handler will be invoked with error message)
+ *
+ * @return an array of <nedges> edge identifiers that sewed togheter
+ *         will build up the input linestring (after snapping). Caller
+ *         will need to free the array using lwfree(), if not null.
+ */
+LWT_ELEMID* lwt_AddLineNoFace(LWT_TOPOLOGY* topo, LWLINE* line, double tol,
+                        int* nedges);
+
+/*
+ * Determine and register all topology faces:
+ *
+ *  - Determines which faces are generated by existing
+ *    edges.
+ *  - Creates face records with correct mbr
+ *  - Update edge left/right face attributes
+ *
+ *  Precondition:
+ *     - the topology edges are correctly linked
+ *     - there are no faces registered in the topology
+ *
+ *  Postconditions:
+ *     - all left/right face attributes of edges
+ *       reference faces with correct mbr.
+ *
+ *  Notes:
+ *     - does not attempt to assign isolated nodes to their
+ *       containing faces
+ *     - does not remove existing face records
+ *     - loads in memory all the topology edges
+ *
+ * @param topo the topology to operate on
+ *
+ * @return 0 on success, -1 on error
+ *         (librtgeom error handler will be invoked with error
+ *         message)
+ */
+int lwt_Polygonize(LWT_TOPOLOGY* topo);
+
 /**
  * Adds a polygon to the topology
  *
@@ -1104,7 +1172,7 @@ void lwt_CreateTopoGeo(LWT_TOPOLOGY* topo, LWGEOM *geom);
  * @param pt the node position
  * @param skipChecks if non-zero skips consistency checks
  *                   (coincident nodes, crossing edges,
- *                    actual face containement)
+ *                    actual face containment)
  *
  * @return ID of the newly added node, or -1 on error
  *         (liblwgeom error handler will be invoked with error message)
@@ -1176,7 +1244,7 @@ LWT_ELEMID lwt_AddIsoEdge(LWT_TOPOLOGY* topo,
  *
  * For ST_AddEdgeModFace
  *
- * If the new edge splits a face, the face is shrinked and a new one
+ * If the new edge splits a face, the face is shrunk and a new one
  * is created. Unless the face being split is the Universal Face, the
  * new face will be on the right side of the newly added edge.
  *
@@ -1186,7 +1254,7 @@ LWT_ELEMID lwt_AddIsoEdge(LWT_TOPOLOGY* topo,
  * @param geom the edge geometry
  * @param skipChecks if non-zero skips consistency checks
  *                   (curve being simple and valid, start/end nodes
- *                    consistency actual face containement)
+ *                    consistency actual face containment)
  *
  * @return ID of the newly added edge or null on error
  *         (liblwgeom error handler will be invoked with error message)
@@ -1209,7 +1277,7 @@ LWT_ELEMID lwt_AddEdgeModFace(LWT_TOPOLOGY* topo,
  * @param geom the edge geometry
  * @param skipChecks if non-zero skips consistency checks
  *                   (curve being simple and valid, start/end nodes
- *                    consistency actual face containement)
+ *                    consistency actual face containment)
  * @return ID of the newly added edge
  *
  */
